@@ -1,5 +1,7 @@
 /*
  *  Copyright (C) 2004 Steve Harris
+ *  Copyright (C) 2006 Garett Shulman
+ *  Copyright (C) 2009 Adam Sampson
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -11,7 +13,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: main.c,v 1.6 2004/07/22 19:52:28 swh Exp $
+ *  $Id: main.c,v 1.7 2005/01/22 17:17:39 swh Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,10 +30,10 @@
 #include <sndfile.h>
 #include <gtk/gtk.h>
 
-#ifdef HAVE_LADCCA
-#include <ladcca/ladcca.h>
+#ifdef HAVE_LASH
+#include <lash/lash.h>
 
-cca_client_t *cca_client;
+lash_client_t *lash_client;
 #endif
 
 #ifdef HAVE_LIBREADLINE
@@ -39,14 +41,16 @@ cca_client_t *cca_client;
 #include <readline/history.h>
 #endif
 
+#ifdef HAVE_LIBLO
+#include <lo/lo.h>
+#endif
+
 #include "threads.h"
 #include "interface.h"
 #include "meters.h"
 #include "support.h"
 #include "main.h"
 
-#define DEFAULT_BUF_LENGTH 10 /* in seconds */
-
 #define DEBUG(lvl, txt...) \
     if (verbosity >= lvl) fprintf(stderr, PACKAGE ": " txt)
 
@@ -64,6 +68,11 @@ char *client_name = DEFAULT_CLIENT_NAME;
 char *prefix = DEFAULT_PREFIX;
 char *format_name = DEFAULT_FORMAT;
 int format_sf = 0;
+int safe_filename = 0;
+int auto_record = 0;
+float auto_begin_threshold = 0.0;
+float auto_end_threshold = 0.0;
+unsigned int auto_end_time = DEFAULT_AUTO_END_TIME;
 
 jack_port_t *ports[MAX_PORTS];
 jack_client_t *client;
@@ -71,6 +80,13 @@ jack_client_t *client;
 GdkPixbuf *img_on, *img_off, *img_busy;
 GdkPixbuf *icon_on, *icon_off;
 
+#ifdef HAVE_LIBLO
+int osc_handler(const char *path, const char *types, lo_arg **argv, int argc,
+		lo_message msg, void *user_data);
+int osc_handler_nox(const char *path, const char *types, lo_arg **argv,
+		int argc, lo_message msg, void *user_data);
+#endif
+
 int main(int argc, char *argv[])
 {
     unsigned int i;
@@ -79,11 +95,15 @@ int main(int argc, char *argv[])
     int console = 0;
     char port_name[32];
     pthread_t dt;
-#ifdef HAVE_LADCCA
-    cca_args_t *cca_args = cca_extract_args(&argc, &argv);
+#ifdef HAVE_LASH
+    lash_args_t *lash_args = lash_extract_args(&argc, &argv);
+     lash_event_t *event;
 #endif
 
-    while ((opt = getopt(argc, argv, "hic:t:n:p:f:")) != -1) {
+    auto_begin_threshold = db2lin(DEFAULT_AUTO_BEGIN_THRESHOLD);
+    auto_end_threshold = db2lin(DEFAULT_AUTO_END_THRESHOLD);
+
+    while ((opt = getopt(argc, argv, "hic:t:n:p:f:sab:e:T:")) != -1) {
 	switch (opt) {
 	case 'h':
 	    help = 1;
@@ -110,6 +130,21 @@ int main(int argc, char *argv[])
 	case 'f':
 	    format_name = optarg;
 	    break;
+	case 's':
+	    safe_filename = 1;
+	    break;
+	case 'a':
+	    auto_record = 1;
+	    break;
+	case 'b':
+	    auto_begin_threshold = db2lin(atof(optarg));
+	    break;
+	case 'e':
+	    auto_end_threshold = db2lin(atof(optarg));
+	    break;
+	case 'T':
+	    auto_end_time = atoi(optarg);
+	    break;
 	default:
 	    num_ports = 0;
 	    break;
@@ -122,7 +157,8 @@ int main(int argc, char *argv[])
 
     if (num_ports < 1 || num_ports > MAX_PORTS || help) {
 	fprintf(stderr, "Usage %s: [-h] [-i] [-c channels] [-n jack-name]\n\t"
-			"[-t buffer-length] [-p file prefix] [-f format] "
+			"[-t buffer-length] [-p file prefix] [-f format]\n\t"
+			"[-a] [-b begin-threshold] [-e end-threshold] [-T end-time]\n\t"
 			"[port-name ...]\n\n", argv[0]);
 	fprintf(stderr, "\t-h\tshow this help\n");
 	fprintf(stderr, "\t-i\tinteractive mode (console instead of X11) also enabled\n\t\tif DISPLAY is unset\n");
@@ -130,7 +166,12 @@ int main(int argc, char *argv[])
 	fprintf(stderr, "\t-n\tspecify the JACK name timemachine will use\n");
 	fprintf(stderr, "\t-t\tspecify the pre-recording buffer length\n");
 	fprintf(stderr, "\t-p\tspecify the saved file prefix, may include path\n");
+	fprintf(stderr, "\t-s\tuse safer characters in filename (windows compatibility)\n");
 	fprintf(stderr, "\t-f\tspecify the saved file format\n");
+	fprintf(stderr, "\t-a\tenable automatic sound-triggered recording\n");
+	fprintf(stderr, "\t-b\tspecify threshold above which automatic recording will begin\n");
+	fprintf(stderr, "\t-e\tspecify threshold below which automatic recording will end\n");
+	fprintf(stderr, "\t-T\tspecify silence length before automatic recording ends\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "\tchannels must be in the range 1-8, default %d\n",
 			DEFAULT_NUM_PORTS);
@@ -138,6 +179,9 @@ int main(int argc, char *argv[])
 	fprintf(stderr, "\tfile-prefix, default \"%s\"\n", DEFAULT_PREFIX);
 	fprintf(stderr, "\tbuffer-length, default %d secs\n", DEFAULT_BUF_LENGTH);
 	fprintf(stderr, "\tformat, default '%s', options: wav, w64\n", DEFAULT_FORMAT);
+	fprintf(stderr, "\tbegin-threshold, default %.1f dB\n", DEFAULT_AUTO_BEGIN_THRESHOLD);
+	fprintf(stderr, "\tend-threshold, default %.1f dB\n", DEFAULT_AUTO_END_THRESHOLD);
+	fprintf(stderr, "\tend-time, default %d secs\n", DEFAULT_AUTO_END_TIME);
 	fprintf(stderr, "\n");
 	fprintf(stderr, "specifying port names to connect to on the command line overrides -c\n\n");
 	exit(1);
@@ -153,11 +197,11 @@ int main(int argc, char *argv[])
 #endif
 
     if (format_sf == 0) {
-	fprintf(stderr, "Unkown format '%s'\n", format_name);
+	fprintf(stderr, "Unknown format '%s'\n", format_name);
     }
 
     /* Register with jack */
-    if ((client = jack_client_new(client_name)) == 0) {
+    if ((client = jack_client_open(client_name, 0, NULL)) == 0) {
 	DEBUG(0, "jack server not running?\n");
 	exit(1);
     }
@@ -165,13 +209,16 @@ int main(int argc, char *argv[])
 
     process_init(buf_length);
 
-#ifdef HAVE_LADCCA
-    cca_client = cca_init (cca_args, "TimeMachine",
-                     0, /* would be CCA_Config_Data_Set etc. */
-                     CCA_PROTOCOL (2,0));
-    if (!cca_client) {
-	DEBUG(1, "could not initialise LADCCA\n");
+#ifdef HAVE_LASH
+    lash_client = lash_init (lash_args, "TimeMachine",
+                     0, /* would be LASH_Config_Data_Set etc. */
+                     LASH_PROTOCOL (2,0));
+    if (!lash_client) {
+	DEBUG(1, "could not initialise LASH\n");
     }
+    event = lash_event_new_with_type(LASH_Client_Name);
+    lash_event_set_string(event, client_name);
+    lash_send_event(lash_client, event);
 #endif
 
     jack_set_process_callback(client, process, 0);
@@ -180,8 +227,8 @@ int main(int argc, char *argv[])
 	DEBUG(0, "cannot activate JACK client");
 	exit(1);
     }
-#ifdef HAVE_LADCCA
-    cca_jack_client_name(cca_client, client_name);
+#ifdef HAVE_LASH
+    lash_jack_client_name(lash_client, client_name);
 #endif
 
     /* Create the jack ports */
@@ -210,6 +257,17 @@ int main(int argc, char *argv[])
 
 #ifdef HAVE_LIBREADLINE
     if (console || !getenv("DISPLAY") || getenv("DISPLAY")[0] == '\0') {
+#ifdef HAVE_LIBLO
+      lo_server_thread st = lo_server_thread_new(OSC_PORT, NULL);
+      if (st) {
+	  lo_server_thread_add_method(st, "/start", "", osc_handler_nox, (void *)1);
+	  lo_server_thread_add_method(st, "/stop", "", osc_handler_nox, (void *)0);
+	  lo_server_thread_start(st);
+	  printf("Listening for OSC requests on osc.udp://localhost:%s\n",
+	    OSC_PORT);
+      }
+#endif
+
       int done = 0;
       while (!done) {
 	char *line = readline("TimeMachine> ");
@@ -252,8 +310,19 @@ int main(int argc, char *argv[])
       bind_meters();
       g_timeout_add(100, meter_tick, NULL);
 
-#ifdef HAVE_LADCCA
-      gtk_idle_add(idle_cb, cca_client);
+#ifdef HAVE_LIBLO
+    lo_server_thread st = lo_server_thread_new(OSC_PORT, NULL);
+    if (st) {
+	lo_server_thread_add_method(st, "/start", "", osc_handler, (void *)1);
+	lo_server_thread_add_method(st, "/stop", "", osc_handler, (void *)0);
+	lo_server_thread_start(st);
+	printf("Listening for OSC requests on osc.udp://localhost:%s\n",
+	       OSC_PORT);
+    }
+#endif
+
+#ifdef HAVE_LASH
+      gtk_idle_add(idle_cb, lash_client);
 #endif
 
       gtk_main();
@@ -283,29 +352,29 @@ void cleanup()
     exit(0);
 }
 
-#ifdef HAVE_LADCCA
+#ifdef HAVE_LASH
 gboolean idle_cb(gpointer data)
 {
-    cca_client_t *cca_client = (cca_client_t *)data;
-    cca_event_t *event;
-    cca_config_t *config;
+    lash_client_t *lash_client = (lash_client_t *)data;
+    lash_event_t *event;
+    lash_config_t *config;
 
-    while ((event = cca_get_event(cca_client))) {
-	if (cca_event_get_type(event) == CCA_Save_Data_Set) {
+    while ((event = lash_get_event(lash_client))) {
+	if (lash_event_get_type(event) == LASH_Save_Data_Set) {
 	    /* we can ignore this as timemachine has no state thats not on the
              * command line */
-	} else if (cca_event_get_type(event) == CCA_Quit) {
+	} else if (lash_event_get_type(event) == LASH_Quit) {
 	    cleanup();
 	} else {
-	    DEBUG(0, "unhandled LADCCA event: type %d, '%s''\n",
-		   cca_event_get_type(event),
-		   cca_event_get_string(event));
+	    DEBUG(0, "unhandled LASH event: type %d, '%s''\n",
+		   lash_event_get_type(event),
+		   lash_event_get_string(event));
 	}
     }
 
-    while ((config = cca_get_config(cca_client))) {
-	DEBUG(0, "got unexpected LADCCA config: %s\n",
-	       cca_config_get_key(config));
+    while ((config = lash_get_config(lash_client))) {
+	DEBUG(0, "got unexpected LASH config: %s\n",
+	       lash_config_get_key(config));
     }
 
     usleep(10000);
